from __future__ import annotations

from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourceAttachments(ResourceAbstract):
    __endpoint: str = "/attachments"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_attachments(self, params: dict = None) -> dict:
        """get_attachments returns a dict with attachment objects"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint, params),
            self.__client.get_basic_headers_for_json()
        )

    def get_attachment(self, attachment_id: str) -> dict:
        """get_attachment returns the referenced (id) attachment"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + attachment_id),
            self.__client.get_basic_headers_for_json())

    def create_attachment(self, payload: bytes) -> dict:
        """create_attachment returns the attachment model as dict on success with the data from the passed payload"""

        return self.__client.upload(
            Helper.create_request_url_from_params(self.__endpoint),
            self.__client.get_basic_headers(),
            payload
        )

    def update_attachment(self, attachment_id: str, payload: dict) -> dict:
        """update_attachment updates the reference (id) attachment with the given payload. Returns a part of the updated attachment model"""

        return self.__client.call(
            "PUT",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                attachment_id),
            self.__client.get_basic_headers_for_json(),
            payload)

    def delete_attachment(self, attachment_id: str) -> None:
        """delete_attachment returns None on success and raises an exception if the attachment couldn't be deleted"""

        self.__client.call(
            "DELETE",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                attachment_id),
            self.__client.get_basic_headers())

    def get_content(self, attachment_id: str, headers: dict = None) -> bytes:
        """get_content returns None on success and raises an exception if the attachment couldn't be deleted"""

        return self.__client.download(
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + attachment_id),
            self.__client.get_basic_headers())



from __future__ import annotations

from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourcePostBoxes(ResourceAbstract):
    __endpoint: str = "/post-boxes"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_post_boxes(self, params: dict = None) -> dict:
        """get_post_boxes returns a dict with post box objects"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint, params),
            self.__client.get_basic_headers_for_json()
        )

    def get_post_box(self, post_box_id: str) -> dict:
        """get_post_box returns the referenced (id) post box"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + post_box_id),
            self.__client.get_basic_headers_for_json())

    def delete_post_box(self, post_box_id: str) -> None:
        """delete_post_box returns None on success and raises an exception if the post box couldn't be deleted"""

        self.__client.call(
            "DELETE",
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + post_box_id),
            self.__client.get_basic_headers())



from __future__ import annotations

from enum import Enum
from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class SendMethod(Enum):
    EMAIL = "email"
    FAX = "fax"
    POST = "post"


class ResourceDocuments(ResourceAbstract):
    __endpoint: str = "/documents"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_documents(self, params: dict = None) -> dict:
        """get_documents returns a dict with document objects"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint, params),
            self.__client.get_basic_headers_for_json()
        )

    def get_document(self, document_id: str) -> dict:
        """get_document returns the referenced (id) document"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + document_id),
            self.__client.get_basic_headers_for_json())

    def create_document(self, payload: dict) -> dict:
        """create_document returns the document model as dict on success with the data from the passed payload"""

        return self.__client.call(
            "POST",
            Helper.create_request_url_from_params(self.__endpoint),
            self.__client.get_basic_headers_for_json(),
            payload
        )

    def update_document(self, document_id: str, payload: dict, params: dict = None) -> dict:
        """update_document updates the reference (id) document with the given payload. Returns the updated document"""

        return self.__client.call(
            "PUT",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_id,
                params
            ),
            self.__client.get_basic_headers_for_json(),
            payload)

    def delete_document(self, document_id: str) -> None:
        """delete_document returns None on success and raises an exception if the document couldn't be deleted"""

        self.__client.call(
            "DELETE",
            Helper.create_request_url_from_params(
                self.__endpoint + "/" + document_id),
            self.__client.get_basic_headers())

    def finalize_document(self, document_id: str) -> dict:
        """finalize_document returns the finalized document on success"""

        return self.__client.call(
            "PUT",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_id +
                "/done"),
            self.__client.get_basic_headers_for_json())

    def cancel_document(self, document_id: str) -> dict:
        """cancel_document returns the canceled document on success"""

        return self.__client.call(
            "POST",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_id +
                "/cancel"),
            self.__client.get_basic_headers_for_json())

    def send_document(
            self,
            document_id: str,
            send_method: SendMethod,
            payload: dict) -> None:
        """send_document returns None on success and rises exception on failure"""

        self.__client.call(
            "POST",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_id +
                "/send/" +
                str(send_method)),
            self.__client.get_basic_headers_for_json(),
            payload)

    def download_document(self, document_id: str) -> bytes:
        """download_document returns the document as bytes on success"""

        return self.__client.download(
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_id +
                "/pdf"),
            self.__client.get_basic_headers_for_pdf(),
        )



import requests

from easybill_rest.resources.resource_attachments import ResourceAttachments
from easybill_rest.resources.resource_contacts import ResourceContacts
from easybill_rest.resources.resource_customer_groups import ResourceCustomerGroups
from easybill_rest.resources.resource_customers import ResourceCustomers
from easybill_rest.resources.resource_discount_position_groups import ResourceDiscountPositionGroups
from easybill_rest.resources.resource_discount_positions import ResourceDiscountPositions
from easybill_rest.resources.resource_document_payments import ResourceDocumentPayments
from easybill_rest.resources.resource_documents import ResourceDocuments
from easybill_rest.resources.resource_logins import ResourceLogins
from easybill_rest.resources.resource_pdf_templates import ResourcePdfTemplates
from easybill_rest.resources.resource_position_groups import ResourcePositionGroups
from easybill_rest.resources.resource_positions import ResourcePositions
from easybill_rest.resources.resource_post_boxes import ResourcePostBoxes
from easybill_rest.resources.resource_projects import ResourceProjects
from easybill_rest.resources.resource_sepa_payments import ResourceSepaPayments
from easybill_rest.resources.resource_serial_numbers import ResourceSerialNumbers
from easybill_rest.resources.resource_stocks import ResourceStocks
from easybill_rest.resources.resource_tasks import ResourceTasks
from easybill_rest.resources.resource_text_templates import ResourceTextTemplates
from easybill_rest.resources.resource_time_trackings import ResourceTimeTrackings
from easybill_rest.resources.resource_webhooks import ResourceWebhooks


class Client:
    _version: str = "0.3.2"
    _base_url: str = "https://api.easybill.de"
    _requests = requests

    api_key: str = ""
    timeout: int

    def __init__(self, api_key: str, timeout: int = 10) -> None:
        self.timeout = timeout
        self.api_key = api_key
        self._requests = requests

    def get_basic_headers(self) -> dict:
        """get_basic_headers returns the basic headers used by the client. Contains auth and
        agent. """

        return {
            "Authorization": "Bearer " + self.api_key,
            "User-Agent": "py-ebrest " + self._version
        }

    def get_basic_headers_for_json(self) -> dict:
        """get_basic_headers_for_json returns the basic headers extended with a
        json content type."""

        return {
            **self.get_basic_headers(),
            **{'Content-type': 'application/json'}
        }

    def get_basic_headers_for_pdf(self) -> dict:
        """get_basic_headers_for_pdf returns the basic headers extended with a pdf content type."""

        return {
            **self.get_basic_headers(),
            **{'Content-type': 'application/pdf'}
        }

    def documents(self) -> ResourceDocuments:
        """documents returns the documents resource which exposes the api documents resource."""

        return ResourceDocuments(self)

    def document_payments(self) -> ResourceDocumentPayments:
        """document_payments returns the document payments resource which exposes the ap
        i document payments resource."""

        return ResourceDocumentPayments(self)

    def customers(self) -> ResourceCustomers:
        """customers returns the customers resource which exposes the api customers resource."""

        return ResourceCustomers(self)

    def customer_groups(self) -> ResourceCustomerGroups:
        """customer_groups returns the customers groups resource which exposes the
         api customer groups resource."""

        return ResourceCustomerGroups(self)

    def positions(self) -> ResourcePositions:
        """positions returns the positions resource which exposes the api positions resource."""

        return ResourcePositions(self)

    def position_groups(self) -> ResourcePositionGroups:
        """position_groups returns the position groups resource which exposes
        the api position groups resource."""

        return ResourcePositionGroups(self)

    def logins(self) -> ResourceLogins:
        """logins returns the logins resource which exposes the api logins resource."""

        return ResourceLogins(self)

    def pdf_templates(self) -> ResourcePdfTemplates:
        """pdf_templates returns the pdf-templates resource which exposes the api
        pdf-templates resource."""

        return ResourcePdfTemplates(self)

    def post_boxes(self) -> ResourcePostBoxes:
        """post_boxes returns the post box resource which exposes the api post-box resource."""

        return ResourcePostBoxes(self)

    def projects(self) -> ResourceProjects:
        """projects returns the projects resource which exposes the api projects resource."""

        return ResourceProjects(self)

    def tasks(self) -> ResourceTasks:
        """tasks returns the tasks resource which exposes the api tasks resource."""

        return ResourceTasks(self)

    def time_trackings(self) -> ResourceTimeTrackings:
        """time_trackings returns the time tracking resource which exposes the
        api time tracking resource."""

        return ResourceTimeTrackings(self)

    def stocks(self) -> ResourceStocks:
        """stocks returns the stocks resource which exposes the api stocks resource."""

        return ResourceStocks(self)

    def serial_numbers(self) -> ResourceSerialNumbers:
        """serial_numbers returns the serial number resource which exposes the api serial number
        resource."""

        return ResourceSerialNumbers(self)

    def sepa_payments(self) -> ResourceSepaPayments:
        """sepa_payments returns the sepa payments resource which exposes the sepa payment
        resource."""

        return ResourceSepaPayments(self)

    def webhooks(self) -> ResourceWebhooks:
        """webhooks returns the webhooks resource which exposes the webhook resource."""

        return ResourceWebhooks(self)

    def contacts(self) -> ResourceContacts:
        """contacts returns the contacts resource which exposes the contact resource."""

        return ResourceContacts(self)

    def text_templates(self) -> ResourceTextTemplates:
        """text_templates returns the text template resource which exposes the text template
        resource."""

        return ResourceTextTemplates(self)

    def discount_positions(self) -> ResourceDiscountPositions:
        """discount_positions returns the position discount resource which exposes the position
        discount resource."""

        return ResourceDiscountPositions(self)

    def discount_position_groups(self) -> ResourceDiscountPositionGroups:
        """discount_position_groups returns the position group discount resource which exposes
        the position group discount resource."""

        return ResourceDiscountPositionGroups(self)

    def attachments(self) -> ResourceAttachments:
        """attachments returns the attachments resource which exposes the attachment resource."""

        return ResourceAttachments(self)

    def call(
            self,
            method: str,
            request_url: str,
            headers: dict,
            passed_payload: dict = None) -> dict:
        """
            call calls the easybill api with the prepared connection.
            :raises: RequestException
        """

        if method == "GET":
            response = self._requests.request(
                method,
                self._base_url + request_url,
                headers=headers,
                params=passed_payload,
                timeout=self.timeout
            )
            response.raise_for_status()

            return response.json()

        if method in ("PUT", "POST"):
            response = self._requests.request(
                method,
                self._base_url + request_url,
                headers=headers,
                json=passed_payload,
                timeout=self.timeout
            )
            response.raise_for_status()

            return response.json()

        if method == "DELETE":
            response = self._requests.request(
                method,
                self._base_url + request_url,
                headers=headers,
                timeout=self.timeout
            )

            response.raise_for_status()
            return {}

        response = self._requests.request(
            method,
            self._base_url + request_url,
            headers=headers,
            timeout=self.timeout
        )
        response.raise_for_status()

        return response.json()

    def upload(self, request_url: str, headers: dict, file: bytes) -> dict:
        """
            upload calls the easybill api with the prepared connection to upload the
            passed bytes / file.
            :raises: RequestException
        """

        response = self._requests.post(
            self._base_url + request_url,
            headers=headers,
            timeout=self.timeout,
            files={'file': file}
        )
        response.raise_for_status()

        return response.json()

    def download(self, request_url: str, headers: dict) -> bytes:
        """
            download calls the easybill api with the prepared connection to download the
            referenced attachment.
            :raises: RequestException
        """

        response = self._requests.get(
            self._base_url + request_url,
            headers=headers,
            timeout=self.timeout)
        response.raise_for_status()

        return response.content



from __future__ import annotations

from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourceDocumentPayments(ResourceAbstract):
    __endpoint: str = "/document-payments"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_document_payments(self, params: dict = None) -> dict:
        """get_document_payments returns a dict with document payments objects"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint, params),
            self.__client.get_basic_headers_for_json()
        )

    def get_document_payment(self, document_payment_id: str) -> dict:
        """get_document_payment returns the referenced (id) document payment"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_payment_id),
            self.__client.get_basic_headers_for_json())

    def create_document_payment(self, payload: dict) -> dict:
        """create_document_payment returns the document payment model as dict on success with the data from the passed payload"""

        return self.__client.call(
            "POST",
            Helper.create_request_url_from_params(self.__endpoint),
            self.__client.get_basic_headers_for_json(),
            payload
        )

    def delete_document_payment(self, document_payment_id: str) -> None:
        """delete_document_payment returns None on success and raises an exception if the document payment couldn't be deleted"""

        self.__client.call(
            "DELETE",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                document_payment_id),
            self.__client.get_basic_headers())



from __future__ import annotations

from abc import ABC
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourceAbstract(ABC):

    def __init__(self) -> None:
        super().__init__()

    @property
    def __endpoint(self) -> str:
        raise NotImplementedError

    @property
    def __client(self) -> Client:
        raise NotImplementedError

    def get_resource_endpoint(self):
        return self.__endpoint



from urllib.parse import urlencode


class Helper:
    @staticmethod
    def create_request_url_from_params(
            endpoint: str, params: dict = None) -> str:
        """create_request_url_from_params is a helper function which is used by the
        resources to generate the request_url"""

        if params is None:
            return "/rest/v1" + endpoint

        return "/rest/v1" + endpoint + "?" + urlencode(params)



import setuptools
from setuptools import setup

with open("readme.md", "r") as fh:
    long_description = fh.read()

setup(
    name="easybill_rest",
    author="Jan Noehles (bolZer)",
    include_package_data=True,
    author_email="noehles@easybill.de",
    description="easybill_rest is a library to work with the easybill REST API (https://www.easybill.de/api/)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/BolZer/py-ebrest",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.7.0',
    version="0.3.2",
    install_requires=[
            "requests",
            "nose<2",
    ],
    extras_require={
        'dev': [
            "coverage<5",
            "wheel",
        ]})



from __future__ import annotations

from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourceStocks(ResourceAbstract):
    __endpoint: str = "/stocks"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_stocks(self, params: dict = None) -> dict:
        """get_stocks returns a dict with stock objects"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint, params),
            self.__client.get_basic_headers_for_json()
        )

    def get_stock(self, stock_id: str) -> dict:
        """get_stock returns the referenced (id) stock object"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(self.__endpoint + "/" + stock_id),
            self.__client.get_basic_headers_for_json()
        )

    def create_stock(self, payload: dict) -> dict:
        """create_stock returns the stock model as dict on success with the data from the passed payload"""

        return self.__client.call(
            "POST",
            Helper.create_request_url_from_params(self.__endpoint),
            self.__client.get_basic_headers_for_json(),
            payload
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from easybill_rest.helper import Helper
from easybill_rest.resources.resource_abstract import ResourceAbstract

if TYPE_CHECKING:
    from easybill_rest import Client


class ResourceContacts(ResourceAbstract):
    __endpoint: str = "/customers"
    __client: Client

    def __init__(self, client: Client) -> None:
        super().__init__()
        self.__client = client

    def get_resource_endpoint(self):
        return self.__endpoint

    def get_contacts(self, customer_id: str, params: dict = None) -> dict:
        """get_contacts returns a dict with contact objects for the referenced customer"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                customer_id +
                "/" +
                "contacts",
                params),
            self.__client.get_basic_headers_for_json())

    def get_contact(self, customer_id: str, contact_id: str) -> dict:
        """get_contact returns the referenced (id) contact for the referenced customer"""

        return self.__client.call(
            "GET",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                customer_id +
                "/" +
                "contacts" +
                "/" +
                contact_id),
            self.__client.get_basic_headers_for_json())

    def create_contact(self, customer_id: str, payload: dict) -> dict:
        """create_contact returns the contact model as dict on success with the data from the passed payload for the referenced customer"""

        return self.__client.call(
            "POST",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                customer_id +
                "/" +
                "contacts"),
            self.__client.get_basic_headers_for_json(),
            payload)

    def update_contact(
            self,
            customer_id: str,
            contact_id: str,
            payload: dict) -> dict:
        """update_contact updates the reference (id) contact with the given payload. Returns the updated contact model"""

        return self.__client.call(
            "PUT",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                customer_id +
                "/" +
                "contacts" +
                "/" +
                contact_id),
            self.__client.get_basic_headers_for_json(),
            payload)

    def delete_contact(self, customer_id: str, contact_id: str) -> None:
        """delete_contact returns None on success and raises an exception if the contact couldn't be deleted"""

        self.__client.call(
            "DELETE",
            Helper.create_request_url_from_params(
                self.__endpoint +
                "/" +
                customer_id +
                "/" +
                "contacts" +
                "/" +
                contact_id),
            self.__client.get_basic_headers())
